--[[ TODO:
	-GUI anpassen
		- Bosse als Tree
		- Items als Tree
		- Spieler als Tree
		- Alles editierbar
]]--

local SynergyRT = LibStub("AceAddon-3.0"):NewAddon("SynergyRT", "AceConsole-3.0", "AceEvent-3.0")

local AceGUI = LibStub("AceGUI-3.0")
local BB = LibStub("LibBabble-Boss-3.0"):GetUnstrictLookupTable()
local LDBIcon = LibStub("LibDBIcon-1.0", true)
local LDB = LibStub("LibDataBroker-1.1", true)

local selectedEvent = nil
local selectedOverview = nil
local selectedItem = nil

local isDst = false

local itemID = nil
local LinkBox = nil

Orig_ChatEdit_InsertLink = ChatEdit_InsertLink
function ChatEdit_InsertLink(text)
  if text and LinkBox and LinkBox:IsVisible() then
    LinkBox:SetText(text)
	itemID = text
    return true
  else
    Orig_ChatEdit_InsertLink(text)
  end
end

-- Those are ignored (Marks, Emblems, Gems, etc.)
local ignoredItems = {
	[40752] = true, -- Emblem of Heroism
	[40753] = true, -- Emblem of Valor
	[45624] = true, -- Emblem of Conquest
	[34057] = true -- Abyss Crystal
	
}

-- Ignored as well (Sartharion Adds)
local ignoredBosses = {
	[BB["Tenebron"]]= true,
	[BB["Shadron"]]= true,
	[BB["Vesperon"]]= true,
	[BB["Stalagg"]]= true,
	[BB["Feugen"]]= true,
	[BB["Death Knight Understudy"]]= true
}

SynergyRT.options = {
	desc = "Synergy Raid Tracker",
	type = "group",
	args = {
		show = {
			name = "show",
			desc = "Show the SynergyRT window",
			type = "execute",
			func = function()
				SynergyRT:ShowGUI()
			end,
		},
		clear = {
			name = "clear",
			desc = "Clear all data",
			type = "execute",
			func = function()
				SynergyRT:ClearCheck()
			end,
		},
		difficulty = {
			name = "difficulty",
			desc = "Needed dungeon difficulty. (Heroic = true, doesn't matter = false",
			type = "toggle",
			get = function(info) return SynergyRT.db.profile.difficulty end,
			set = function(info,val) SynergyRT.db.profile.difficulty = val SynergyRT:Print("Difficulty set to: "..tostring(val)) end,
		},
		members = {
			name = "members",
			desc = "Minimum Number of Raidmembers",
			type = "input",
			set = function(info,val)
						SynergyRT.db.profile.members = tonumber(val)
						SynergyRT:Print("Members set to: "..tostring(val))
						SynergyRT:RAID_ROSTER_UPDATE()
			end
		},
		minimap = {
			name = "minimap",
			desc = "Toggle Minimap Icon",
			type = "toggle",
			get = function(info) return SynergyRT.db.profile.minimap end,
			set = function(info,val)
						SynergyRT.db.profile.minimap = val
						if val then
							LDBIcon:Show("SynergyRT")
						else
							LDBIcon:Hide("SynergyRT")
						end
			end,
		},
		popup = {
			name = "popup",
			desc = "Toggle Loot-DKP Popup",
			type = "toggle",
			get = function(info) return SynergyRT.db.profile.popup end,
			set = function(info,val) SynergyRT.db.profile.popup = val SynergyRT:Print("Popup set to: "..tostring(val)) end,
		},
		standby = {
			type = "toggle",
			name = "Standby",
			desc = "Toggles SynergyRT on/off",
			get = function(info) return SynergyRT.db.profile.standby end,
			set = function(info, val)
				SynergyRT.db.profile.standby = val
				if val then
					SynergyRT:Disable()
				else
					SynergyRT:Enable()					
				end
			end,
		},
		wipe = {
			name = "wipe",
			desc = "Toggle WipeCheck",
			type = "toggle",
			get = function(info) return SynergyRT.db.profile.wipe end,
			set = function(info,val) SynergyRT.db.profile.wipe = val SynergyRT:Print("WipeCheck set to: "..tostring(val)) end,
		},
	},
}

local defaults = {
	profile = {
		events = {},
		popup = true,
		difficulty = true,
		raidActive = nil,
		members = 11,
		standby = false,
		minimap = false,
		minimapIcon = {},
		attendees = {}
	}
}

function SynergyRT:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("SynergyRTDB", defaults, "profile")
	LibStub("AceConfig-3.0"):RegisterOptionsTable("SynergyRT", self.options)
	LibStub("AceConfigCmd-3.0"):CreateChatCommand("synergyrt", "SynergyRT")
	
	if LDB then
		self.ldb = LDB:NewDataObject("SynergyRT",
				{
					type =  "launcher", 
					label = "SynergyRT\n"..
							"|cffffff00Left click|r to open the GUI.\n"..
							"|cffffff00Right click|r to clear tracked data.",
					icon = "Interface\\Addons\\SynergyRT\\icon",
					OnClick = function(clickedframe, button)
						if button == "LeftButton" then
							self:ShowGUI()
						elseif button == "RightButton" then
							self:ClearCheck()
						end
					end,
				})
		if LDBIcon then
			LDBIcon:Register("SynergyRT", self.ldb, self.db.profile.minimapIcon)
			if not self.db.profile.minimap then
				LDBIcon:Hide("SynergyRT")
			end
		end
	end
	
	if self.db.raidActive then
		self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
		self:RegisterEvent("CHAT_MSG_LOOT")
		self:Print("is still tracking!")
	end
	
	if self.db.profile.standby then
		self:Disable()
	else
		self:Enable()
	end
end

function SynergyRT:OnEnable()
	self:RegisterEvent("RAID_ROSTER_UPDATE")
	if self.db.profile.wipe then
		self:RegisterEvent("PLAYER_REGEN_ENABLED")
	end
	self:RAID_ROSTER_UPDATE()
	self:Print("is now enabled")
end

function SynergyRT:OnDisable()
	self:UnregisterAllEvents()
	self.db.raidActive = nil
	self:Print("is now disabled")
end

function SynergyRT:RAID_ROSTER_UPDATE()
	local num = GetNumRaidMembers()
	
	local id, inBG = false
	for i=1, MAX_BATTLEFIELD_QUEUES do
		_, _, id = GetBattlefieldStatus(i);
		if( id > 0 ) then
			inBG = true
		end
	end
	if not inBG and ((self.db.profile.difficulty and GetCurrentDungeonDifficulty() == 2) or not self.db.profile.difficulty) then
		-- Only check, if enough players in raid
		if not self.db.raidActive and num >= self.db.profile.members then
			self.db.raidActive = true
			GuildRoster()
			self:Print("Tracking activated.")
			self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
			self:RegisterEvent("CHAT_MSG_LOOT")
			self:CreateEvent()
			for i=1, num do
				local name,_,_,level,class = GetRaidRosterInfo(i)
				local race = UnitRace("raid"..i)
				table.insert(self.db.profile.events[#self.db.profile.events].joins, {['name']=name, ['time']=time()})
				table.insert(self.db.profile.attendees, {['name']=name, ['level']=level, ['class']=class, ['race']=race})
			end
			self.db.profile.events[#self.db.profile.events].attendees = self.db.profile.attendees
		elseif self.db.raidActive and num < self.db.profile.members then
			self.db.raidActive = nil
			self:Print("Tracking deactivated.")
			self:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
			self:UnregisterEvent("CHAT_MSG_LOOT")
			self.db.profile.events[#self.db.profile.events].endTime = time()
			self.db.profile.attendees = {}
		elseif self.db.raidActive and num > self.db.profile.members then
			local currentAttendees={}
			for i=1, num do
				local name,_,_,level,class = GetRaidRosterInfo(i)
				local race = UnitRace("raid"..i)
				table.insert(currentAttendees, {['name']=name, ['level']=level, ['class']=class, ['race']=race})
			end
			for i,c in currentAttendees do
				local found = false
				for u, s in self.db.profile.attendees do
					if s.name == c.name then
						found = true
						break
					end
				end
				if not found then
					table.insert(self.db.profile.events[#self.db.profile.events].joins, {['name']=c.name,['time']=time()})
					table.insert(self.db.profile.attendees, c)
					table.insert(self.db.profile.events[#self.db.profile.events].attendees, c)
				end
			end
			for i,s in self.db.profile.attendees do
				local found = false
				for u, c in currentAttendees do
					if s.name == c.name then
						found = true
						break
					end
				end
				if not found then
					table.insert(self.db.profile.events[#self.db.profile.events].leaves, {['name']=c.name,['time']=time()})
					table.remove(self.db.profile.attendees, i)
				end
			end
		end
	end
end

local function TimeAsUnix(timestr)
	if not timestr then return end
	local datestr, timestr = strsplit(" ", timestr)
	local month, day, year = strsplit("/", datestr)
	local hour, min, sec = strsplit(":", timestr)
	return time({month=month, day=day, year=year, hour=hour, min=min, sec=sec, isdst=isDst})
end

local function FromUnixTime(timestamp)
	if not timestamp then return end
	local dt = date("*t", tonumber(timestamp))
	isDst = dt.isdst
	return string.format("%02d/%02d/%d %02d:%02d:%02d", dt.month, dt.day, dt.year, dt.hour, dt.min, dt.sec)
end

function SynergyRT:AddEvent()	
	local event = {}
	event.key = FromUnixTime(time())
	event.realm = GetRealmName()
	event.zone = GetRealZoneText()
	event.note = nil
	event.startTime = time()
	event.endTime = nil
	event.attendees = {}
	event.loots = {}
	event.bosses = {}
	event.wipes = {}
	event.joins = {}
	event.leaves = {}
	table.insert(self.db.profile.events, event)
end

function SynergyRT:AddBoss(mob)
	local boss = {
		name = mob,
		time = time(),
		attendees = self.db.profile.attendees
	}
	
	table.insert(self.db.profile.events[#self.db.profile.events].bosses, boss)
end

function SynergyRT:AddWipe()
	table.insert(self.db.profile.events[#self.db.profile.events].wipes, {['time']=time()})
end

function SynergyRT:PLAYER_REGEN_ENABLED()
	local BigWigs = _G["BigWigs"]
	if not UnitIsFeignDeath("player") then
		local go = BigWigs.modulePrototype:Scan()
		if not go then
			self:IsBossWipe()
			return
		end
	end

	if not UnitAffectingCombat("player") then
		self:ScheduleEvent(self.PLAYER_REGEN_ENABLED, 2, self)
	end
end

local linkpat = '|c........|Hitem:(%%d+):.-|h|r'
local globalpatterns = {
   "LOOT_ITEM",
   "LOOT_ITEM_SELF",
}
local patterns = {}
for _, global in ipairs(globalpatterns) do
   local pattern = _G[global]
   if strsub(pattern, 1,2) == "%s" then
      pattern = "([^ ]+)"..strsub(pattern, 3)
   end
   patterns[global] = gsub(gsub(pattern, "%%d", "(%%d+)"), "%%s", linkpat)
end

-- Check for Loot
function SynergyRT:CHAT_MSG_LOOT(_,msg)
   player, item = strmatch(msg, patterns["LOOT_ITEM"])
   if player and item then
      self:ItemLooted(player, item)
      return
   end

   item = strmatch(msg, patterns["LOOT_ITEM_SELF"])
   if item then
      self:ItemLooted(UnitName("player"), item)
      return
   end
end

-- Check for Bosskill
function SynergyRT:COMBAT_LOG_EVENT_UNFILTERED(_, _, event, _, _, _, _, mob)
	if event ~= "UNIT_DIED" and event ~= "UNIT_DESTROYED" then return end
	if mob and not ignoredBosses[mob] and BB[mob] then
		self:Print(format("%s down. Erfasse Anwesenheit...",mob))
		self:AddBoss(mob)
	end
end

-- Clear all Data
function SynergyRT:ClearAll()
	self.db.profile.events = {}
	
	selectedEvent=nil
	selectedOverview=nil
	selectedItem=nil
	
	self:Print("All data cleared!")
end

-- Get Item and save into DB
function SynergyRT:ItemLooted(player, item)
	local _, itemLink, itemRarity = GetItemInfo(item)
	
	local itemID = select(3, itemLink:find("item:(%d+):"))
	if not itemID then return false end
	itemID = tonumber(itemID:trim())
	if type(itemID) ~= "number" then return false end
	if ignoredItems[itemID] or (itemRarity < 4 and not ignoredItems[itemID]) then return false end
	
	local loot = {
		player = player,
		item = itemID,
		dkp = 0,
		time = time(),
		boss = nil,
		zone = GetRealZoneText(),
		note = nil
	}
	
	if difftime(loot.time,self.db.profile.events[#self.db.profile.events].bosses[#self.db.profile.events[#self.db.profile.events].bosses].time) < 300 then
		loot.boss = self.db.profile.events[#self.db.profile.events].bosses[#self.db.profile.events[#self.db.profile.events].bosses].name
	else
		loot.boss = "Trash"
	end
	
	table.insert(self.db.profile.events[#self.db.profile.events].loots, loot)
	
	if self.db.profile.popup then
		self:GetDKP()
	end
end

-- Convert selected kill to EQDKP-String
function SynergyRT:CreateExportString(event)
	if not event.endTime then return "Event has no Endtime!" end
	local text = "<RaidInfo><Version>1.4</Version>"

	text = text .. "<key>" .. FromUnixTime(event.startTime) .. "</key>"
	text = text .. "<realm>" .. event.realm .. "</realm>"
	text = text .. "<start>" .. FromUnixTime(event.startTime) .. "</start>"
	text = text .. "<end>" .. FromUnixTime(event.endTime) .. "</end>"
	text = text .. "<zone>" .. event.zone .. "</zone>"
	if event.note then
		text = text .. "<note><![CDATA[" .. event.note .. " - Zone: " ..  event.zone .. "]]></note>"
	end
	text = text .. "<exporter>" .. UnitName("Player") .. "</exporter>"

	text = text .. "<PlayerInfos>"
	
	for i, v in ipairs(event.attendees) do
		text = text .. "<key"..i.."><name>" .. v.name .. "</name>"
		if v.class then text = text .. "<class>" .. v.class .. "</class>" end
		if v.level then text = text .. "<level>" .. v.level .. "</level>" end
		if v.race then text = text .. "<race>" .. v.race .. "</race>" end
		if v.note then text = text .. "<note><![CDATA["..v.note.."]]></note>" end
		text = text .. "</key"..i..">"
	end

	text = text .. "</PlayerInfos>"

	text = text .. "<BossKills>"
	for i,boss in ipairs(event.bosses) do
		text = text .. "<key"..i.."><name>" .. boss.name .. "</name><time>" .. FromUnixTime(boss.time) .. "</time><attendees>"
		for k,v in boss.attendees do
			text = text .. "<key"..k.."><name>"..v.."</name></key"..k..">"
		end
		text = text .. "</attendees></key"..i..">"
	end
	text = text .. "</BossKills>"
	
	text = text .. "<Wipes>"
	for i,wipe in ipairs(event.wipes) do
		text = text .. "<Wipe>" .. FromUnixTime(wipe.time) .. "</Wipe>"
	end
	text = text .. "</Wipes>"
	
	local players_join = ""
	local players_leave = ""
	
	for i,join in ipairs(event.joins) do
		players_join = players_join .. "<key"..i.."><player>" .. join.name .. "</player><time>" .. FromUnixTime(join.time) .."</time></key"..i..">"
	end
	for i,leave in ipairs(event.leaves) do
		players_leave = players_leave .. "<key"..i.."><player>" .. leave.name .. "</player><time>" .. FromUnixTime(leave.time) .."</time></key"..i..">"
	end
	
	text = text .. "<Join>" .. players_join .. "</Join>"
	text = text .. "<Leave>" .. players_leave .. "</Leave>"

	text = text .. "<Loot>"


	for i, item in ipairs(event.loots) do
		local itemName,_,rarity = GetItemInfo(item.item)
		local _,_,_,hex = GetItemQualityColor(rarity)

		text = text .. "<key"..i..">"
		text = text .. "<ItemName>" .. (itemName or "?") .. "</ItemName>"
		text = text .. "<ItemID>" .. item.item .. "</ItemID>"
		text = text .. "<Count>1</Count>"
		text = text .. "<Color>" .. strsub(hex,3) .. "</Color>"
		text = text .. "<Player>" .. item.player .. "</Player>"
		text = text .. "<Costs>" .. item.dkp .. "</Costs>"
		text = text .. "<Time>" .. FromUnixTime(item.time) .. "</Time>"
		text = text .. "<Zone>" .. item.zone .. "</Zone>"
		text = text .. "<Boss>" .. item.boss .. "</Boss>"
		if item.note then
			text = text .. "<Note><![CDATA[" .. item.note .. " - Zone: " .. item.zone .. " - Boss: " .. item.boss .. " - " .. item.dkp .." DKP]]</Note>"
		end
		text = text .. "</key"..i..">"
	end

	text = text .. "</Loot>"
	text = text .. "</RaidInfo>"
	return text
end

function SynergyRT:ShowGUI()
	if self.frame and self.frame:IsShown() then
		AceGUI:Release(self.frame)
		self.frame = nil
	end

	local fMain = AceGUI:Create("Frame")
	fMain:SetTitle("SynergyRT Menu")
	fMain:SetLayout("List")
	fMain:SetWidth(800)
	fMain:SetHeight(600)
	fMain:SetCallback("OnClose", function(widget) AceGUI:Release(widget) self.frame = nil end)
	fMain:Show()
	
	self.frame = fMain
	
	local tabEvents = AceGUI:Create("TabGroup")
	tabEvents:SetLayout("Fill")
	tabEvents:SetFullWidth(true)
	tabEvents:SetHeight(400)
	local tabs = {}
	for i,v in ipairs(self.db.profile.events) do
		table.insert(tabs, {value = i, text = v.key})
	end
	tabEvents:SetTabs(tabs)
	tabEvents:SetCallback("OnGroupSelected", function(widget,event,value) self:SelectEvent(widget,value) end )
	
	local bNew = AceGUI:Create("Button")
	bNew:SetText("Create new Event")
	bNew:SetWidth(200)
	bNew:SetCallback("OnClick", function()
									self:AddEvent()
									self:ShowGUI()
								end)
	
	fMain:SetStatusText(#tabs .." entries found.")

	fMain:AddChild(bNew)
	fMain:AddChild(tabEvents)
	tabEvents:SelectTab(selectedEvent)
end

function SynergyRT:DrawOverview(widget)
	local tabOverview = AceGUI:Create("TabGroup")
	tabOverview:SetLayout("Fill")
	local tabs = {{value="Event", text="Event"},{value="Players", text="Players"},{value="Bosses", text="Bosses"},{value="Wipes", text="Wipes"},{value="Loots", text="Loots"}}
	tabOverview:SetTabs(tabs)
	tabOverview:SetCallback("OnGroupSelected", function(w,e,v) self:SelectOverview(w,e,v) end )
	
	widget:AddChild(tabOverview)
	tabOverview:SelectTab(selectedOverview)
end

function SynergyRT:SelectEvent(widget, value)
	widget:ReleaseChildren()
	if selectedEvent ~= value then
		selectedEvent = value
		selectedOverview = nil
		selectedItem = nil
	end
	self:DrawOverview(widget)
end

function SynergyRT:DrawEvent(widget)
	local sfEvent = AceGUI:Create("ScrollFrame")
	sfEvent:SetLayout("List")
	
	local sgRow1 = AceGUI:Create("SimpleGroup")
	sgRow1:SetLayout("Flow")
	sgRow1:SetFullWidth(true)
	
	local eName = AceGUI:Create("EditBox")
	eName:SetLabel("Key")
	eName:SetWidth(180)
	eName:SetText(self.db.profile.events[selectedEvent].key)
	eName:SetCallback("OnEnterPressed",function(widget,event,text) self.db.profile.events[selectedEvent].key = strtrim(text) self:ShowGUI() end )
	
	local eZone = AceGUI:Create("EditBox")
	eZone:SetLabel("Zone")
	eZone:SetWidth(180)
	eZone:SetText(self.db.profile.events[selectedEvent].zone)
	eZone:SetCallback("OnEnterPressed",function(widget,event,text) widget.editbox:ClearFocus() self.db.profile.events[selectedEvent].zone = strtrim(text) end )
	
	local sgRow2 = AceGUI:Create("SimpleGroup")
	sgRow2:SetLayout("Flow")
	sgRow2:SetFullWidth(true)
	
	local eStart = AceGUI:Create("EditBox")
	eStart:SetLabel("Starttime")
	eStart:SetWidth(180)
	eStart:SetText(FromUnixTime(self.db.profile.events[selectedEvent].startTime))
	eStart:SetCallback("OnEnterPressed",function(widget,event,text) widget.editbox:ClearFocus() self.db.profile.events[selectedEvent].startTime = TimeAsUnix(strtrim(text)) end )
		
	local eEnd = AceGUI:Create("EditBox")
	eEnd:SetLabel("Endtime")
	eEnd:SetWidth(180)
	eEnd:SetText(FromUnixTime(self.db.profile.events[selectedEvent].endTime))
	eEnd:SetCallback("OnEnterPressed",function(widget,event,text) widget.editbox:ClearFocus() self.db.profile.events[selectedEvent].endTime = TimeAsUnix(strtrim(text)) end )
	
	local sgRow3 = AceGUI:Create("SimpleGroup")
	sgRow3:SetLayout("Flow")
	sgRow3:SetFullWidth(true)
	
	local bStart = AceGUI:Create("Button")
	bStart:SetText("Current Time")
	bStart:SetWidth(180)
	bStart:SetCallback("OnClick", function() local t = time() eStart:SetText(FromUnixTime(t)) self.db.profile.events[selectedEvent].startTime = t end)
	
	local bEnd = AceGUI:Create("Button")
	bEnd:SetText("Current Time")
	bEnd:SetWidth(180)
	bEnd:SetCallback("OnClick", function() local t = time() eEnd:SetText(FromUnixTime(t)) self.db.profile.events[selectedEvent].endTime = t end)
	
	local sgControls = AceGUI:Create("SimpleGroup")
	sgControls:SetLayout("Flow")
	sgControls:SetFullWidth(true)
	
	local meExport = AceGUI:Create("MultiLineEditBox")
	meExport:SetLabel("Export-String")
	meExport:SetFullWidth(true)
	meExport:SetText()
	meExport:SetCallback("OnEnterPressed", function(w,e,v) end)
	
	local bExport = AceGUI:Create("Button")
	bExport:SetText("Export Event")
	bExport:SetWidth(180)
	bExport:SetCallback("OnClick", function()
										meExport:SetText(self:CreateExportString(self.db.profile.events[selectedEvent]))
									end)
	
	local bRemove = AceGUI:Create("Button")
	bRemove:SetText("Remove Event")
	bRemove:SetWidth(180)
	bRemove:SetCallback("OnClick", function()
										table.remove(self.db.profile.events, selectedEvent)
										selectedEvent = nil
										selectedOverview = nil
										self:ShowGUI()
									end)
									
	widget:AddChild(sfEvent)
	sfEvent:AddChild(sgRow1)
	sfEvent:AddChild(sgRow2)
	sfEvent:AddChild(sgRow3)
	sfEvent:AddChild(sgControls)
	sfEvent:AddChild(meExport)
	sgRow1:AddChild(eName)
	sgRow1:AddChild(eZone)
	sgRow2:AddChild(eStart)
	sgRow2:AddChild(eEnd)
	sgRow3:AddChild(bStart)
	sgRow3:AddChild(bEnd)
	sgControls:AddChild(bExport)
	sgControls:AddChild(bRemove)
end

function SynergyRT:DrawPlayers(widget)

end

function SynergyRT:DrawBosses(widget)

end

function SynergyRT:DrawWipes(widget)

end

function SynergyRT:DrawLoots(widget)
	local sfLoots = AceGUI:Create("ScrollFrame")
	sfLoots:SetLayout("List")	

	local sgTree = AceGUI:Create("SimpleGroup")
	sgTree:SetLayout("Fill")
	sgTree:SetFullWidth(true)
	sgTree:SetHeight(250)
	
	local treeLoots = AceGUI:Create("TreeGroup")
	treeLoots:SetLayout("Fill")
	local tree = {}
	for i,v in ipairs(self.db.profile.events[selectedEvent].loots) do
		table.insert(tree, {value = i, text = GetItemInfo(v.item)})
	end
	treeLoots:SetTree(tree)
	treeLoots:SetCallback("OnGroupSelected", function(w,e,v) self:SelectItem(w,v) end)

	local sgNew = AceGUI:Create("SimpleGroup")
	sgNew:SetLayout("Flow")
	sgNew:SetFullWidth(true)
	
	local eNew = AceGUI:Create("EditBox")
	eNew:SetLabel("New Item")
	eNew:SetText()
	eNew:SetWidth(200)
	eNew:SetCallback("OnEnterPressed", function(widget, event, text)
											itemID = text
											widget.editbox:ClearFocus()
										end)
	LinkBox = eNew

	local bNew = AceGUI:Create("Button")
	bNew:SetText("Add Item")
	bNew:SetWidth(200)
	bNew:SetCallback("OnClick", function()
									if not itemID then self:Print("No Item set!") return end
									_,itemID = GetItemInfo(itemID)
									if(itemID) then
										_, _, itemID = string.find(itemID, "^|c%x+|%a+:(%d+):.+|h%[.*%]")
										table.insert(self.db.profile.events[selectedEvent].loots, {['item']=tonumber(itemID),['player']="player",['dkp']=0,['time']=time(),['boss']="boss",['zone']=GetRealZoneText()})
										selectedItem = #self.db.profile.events[selectedEvent].loots
										self:ShowGUI()
									else
										self:Print("Invalid Item: "..itemID)
									end
								end)
	
	widget:AddChild(sfLoots)
	sfLoots:AddChild(sgNew)
	sgNew:AddChild(eNew)
	sgNew:AddChild(bNew)
	sfLoots:AddChild(sgTree)
	sgTree:AddChild(treeLoots)
	
	if selectedItem then treeLoots:Select(selectedItem) end
end

function SynergyRT:SelectOverview(widget, event, value)
	widget:ReleaseChildren()
	if selectedOverview ~= value then
		selectedOverview = value
		selectedItem = nil
	end
	if value == "Event" then
		self:DrawEvent(widget)
	elseif value == "Players" then
		self:DrawPlayers(widget)	
	elseif value == "Bosses" then
		self:DrawBosses(widget)
	elseif value == "Wipes" then
		self:DrawWipes(widget)
	elseif value == "Loots" then
		self:DrawLoots(widget)
	end
end

function SynergyRT:SelectItem(widget, value)
	widget:ReleaseChildren()
	if selectedItem ~= value then
		selectedItem = value
	end
	
	local sf = AceGUI:Create("ScrollFrame")
	sf:SetLayout("List")
	
	local sgRow1 = AceGUI:Create("SimpleGroup")
	sgRow1:SetLayout("Flow")
	sgRow1:SetFullWidth(true)
	
	local sgRow2 = AceGUI:Create("SimpleGroup")
	sgRow2:SetLayout("Flow")
	sgRow2:SetFullWidth(true)
	
	local eItemID = AceGUI:Create("EditBox")
	eItemID:SetLabel("ItemID")
	eItemID:SetWidth(180)
	eItemID:SetText(self.db.profile.events[selectedEvent].loots[value].item)
	eItemID:SetCallback("OnEnterPressed",function(widget,event,text) widget.editbox:ClearFocus() self.db.profile.events[selectedEvent].loots[value].item = strtrim(text) end )
	
	local eLooter = AceGUI:Create("EditBox")
	eLooter:SetLabel("Looted by")
	eLooter:SetWidth(180)
	eLooter:SetText(self.db.profile.events[selectedEvent].loots[value].player)
	eLooter:SetCallback("OnEnterPressed",function(widget,event,text) widget.editbox:ClearFocus() self.db.profile.events[selectedEvent].loots[value].player = strtrim(text) end )
	
	local eDKP = AceGUI:Create("EditBox")
	eDKP:SetLabel("DKP")
	eDKP:SetWidth(180)
	eDKP:SetText(self.db.profile.events[selectedEvent].loots[value].dkp)
	eDKP:SetCallback("OnEnterPressed",function(widget,event,text) widget.editbox:ClearFocus() self.db.profile.events[selectedEvent].loots[value].dkp = strtrim(text) end )
	
	local sgTime = AceGUI:Create("SimpleGroup")
	sgTime:SetFullWidth(true)
	sgTime:SetLayout("Flow")
	
	local eTime = AceGUI:Create("EditBox")
	eTime:SetLabel("Time")
	eTime:SetWidth(180)
	eTime:SetText(FromUnixTime(self.db.profile.events[selectedEvent].loots[value].time))
	eTime:SetCallback("OnEnterPressed",function(widget,event,text) self.db.profile.events[selectedEvent].loots[value].time = TimeAsUnix(strtrim(text)) end )
	
	local bTime = AceGUI:Create("Button")
	bTime:SetText("Current Time")
	bTime:SetWidth(180)
	bTime:SetCallback("OnClick", function() local t = time() eTime:SetText(FromUnixTime(t)) self.db.profile.events[selectedEvent].loots[value].time = t end)
	
	local eZone = AceGUI:Create("EditBox")
	eZone:SetLabel("Zone")
	eZone:SetWidth(180)
	eZone:SetText(self.db.profile.events[selectedEvent].loots[value].zone)
	eZone:SetCallback("OnEnterPressed",function(widget,event,text) widget.editbox:ClearFocus() self.db.profile.events[selectedEvent].loots[value].zone = strtrim(text) end )
	
	local eBoss = AceGUI:Create("EditBox")
	eBoss:SetLabel("Boss")
	eBoss:SetWidth(180)
	eBoss:SetText(self.db.profile.events[selectedEvent].loots[value].boss)
	eBoss:SetCallback("OnEnterPressed",function(widget,event,text) widget.editbox:ClearFocus() self.db.profile.events[selectedEvent].loots[value].boss = strtrim(text) end )
	
	local bRemove = AceGUI:Create("Button")
	bRemove:SetText("Remove Item")
	bRemove:SetWidth(180)
	bRemove:SetCallback("OnClick", function()
								table.remove(self.db.profile.events[selectedEvent].loots, value)
								selectedItem = nil
								self:ShowGUI()
							end )
	widget:AddChild(sf)
	sf:AddChild(sgRow1)
	sgRow1:AddChild(eItemID)
	sgRow1:AddChild(eLooter)
	sf:AddChild(eDKP)
	sf:AddChild(sgTime)
	sgTime:AddChild(eTime)
	sgTime:AddChild(bTime)
	sf:AddChild(sgRow2)
	sgRow2:AddChild(eZone)
	sgRow2:AddChild(eBoss)
	sf:AddChild(bRemove)
end

function SynergyRT:GetDKP()
	local lootid = #self.db.profile.events[#self.db.profile.events].loots
	local item = GetItemInfo(self.db.profile.events[#self.db.profile.events].loots[lootid].item)
	local player = self.db.profile.events[#self.db.profile.events].loots[lootid].player
	
	local fDKP = AceGUI:Create("Frame")
	fDKP:SetTitle("DKP eingeben")
	fDKP:SetLayout("List")
	fDKP:SetWidth(400)
	fDKP:SetHeight(200)
	fDKP:SetCallback("OnClose", function(widget) AceGUI:Release(widget) end)
	
	local sg = AceGUI:Create("SimpleGroup")
	
	local lLooter = AceGUI:Create("Label")
	lLooter:SetWidth(300)
	lLooter:SetText(player..": "..item)
	
	local eDKP = AceGUI:Create("EditBox")
	eDKP:SetLabel("DKP:")
	eDKP:SetWidth(300)
	eDKP:SetText("0")
	eDKP:SetCallback("OnEnterPressed",function(widget,event,text)
										widget.editbox:ClearFocus()
										self.db.profile.events[#self.db.profile.events].loots[lootid].dkp = strtrim(text)
									end )
	
	local bDisenchanted = AceGUI:Create("Button")
	bDisenchanted:SetText("disenchanted")
	bDisenchanted:SetWidth(200)
	bDisenchanted:SetCallback("OnClick", function()
								self.db.profile.events[#self.db.profile.events].loots[lootid].player = "disenchanted"
								lLooter:SetText("disenchanted: "..item)
							end )
	local bBank = AceGUI:Create("Button")
	bBank:SetText("bank")
	bBank:SetWidth(200)
	bBank:SetCallback("OnClick", function()
								self.db.profile.events[#self.db.profile.events].loots[lootid].player = "bank"
								lLooter:SetText("bank: "..item)
							end )
	
	sg:AddChild(lLooter)
	sg:AddChild(eDKP)
	sg:AddChild(bDisenchanted)
	sg:AddChild(bBank)
	fDKP:AddChild(sg)
end

function SynergyRT:IsBossWipe()
	fWipe = AceGUI:Create("BlankFrame")
	fWipe:SetTitle("WipeCheck")
	fWipe:SetLayout("List")
	fWipe:SetWidth(200)
	fWipe:SetHeight(100)
	fWipe:SetCallback("OnClose", function(widget) AceGUI:Release(widget) end)
	fWipe:Show()
	
	lQuestion = AceGUI:Create("Label")
	lQuestion:SetWidth(150)
	lQuestion:SetText("Wiped at a Boss?\n(Target the Boss if it is)")
	
	bYes = AceGUI:Create("Button")
	bYes:SetText("Yes")
	bYes:SetWidth(50)
	bYes:SetCallback("OnClick", function()
								self:AddWipe()
								fWipe:Hide()
							end)
	
	bNo = AceGUI:Create("Button")
	bNo:SetText("No")
	bNo:SetWidth(50)
	bNo:SetCallback("OnClick", function()
								fWipe:Hide()
							end)
							
	fWipe:AddChild(lQuestion)
	fWipe:AddChild(bYes)
	fWipe:AddChild(bNo)
end

function SynergyRT:ClearCheck()
	fReset = AceGUI:Create("BlankFrame")
	fReset:SetTitle("Reset")
	fReset:SetLayout("List")
	fReset:SetWidth(200)
	fReset:SetHeight(100)
	fReset:SetCallback("OnClose", function(widget) AceGUI:Release(widget) end)
	fReset:Show()
	
	lQuestion = AceGUI:Create("Label")
	lQuestion:SetWidth(150)
	lQuestion:SetText("Wirklich alle Daten l\195\182schen?")
	
	sgControls = AceGUI:Create("SimpleGroup")
	sgControls:SetLayout("Flow")
	
	bYes = AceGUI:Create("Button")
	bYes:SetText("Yes")
	bYes:SetWidth(50)
	bYes:SetCallback("OnClick", function()
								self:ClearAll()
								fReset:Hide()
							end)
	
	bNo = AceGUI:Create("Button")
	bNo:SetText("No")
	bNo:SetWidth(50)
	bNo:SetCallback("OnClick", function()
								fReset:Hide()
							end)
	
	fReset:AddChild(lQuestion)
	sgControls:AddChild(bYes)
	sgControls:AddChild(bNo)
	fReset:AddChild(sgControls)
end
